* Program....: SUMMARIZE.PRG
* Version....: 1.0
* Author.....: Andrew Coates
* Date.......: July 4, 1999
* Notice.....: Copyright  1999 Civil Solutions
*   All Rights Reserved.
* Compiler...: Visual FoxPro 06.00.8492.00 for Windows
* Abstract...: Calculates the balances of all records
*  including the sum of allsubordinate nodes for a
*  non-recursive, connected, hierarchical data set
* Changes....:

#INCLUDE FOXPRO.h
#DEFINE MAX_LEVELS  20
#DEFINE MAX_LEVELS_STRING_LOC  ;
  "Maximum number of levels exceeded - " + ;
  "there is probably a recursive heirarchy"
#DEFINE MAX_LEVELS_CAPTION_LOC  ;
  "Heirarchy Summary"

* create a record set to play with
* you'd probably have one open in the appropriate
* work area, or pass its name to this function
CREATE CURSOR t0 (AC C(6), Bal Y, PAC C(6))
INSERT INTO t0 VALUES ('1-0000',$0,'')
INSERT INTO t0 VALUES ('1-1000',$0,'1-0000')
INSERT INTO t0 VALUES ('1-1100',$200,'1-1000')
INSERT INTO t0 VALUES ('1-1200',$350,'1-1000')
INSERT INTO t0 VALUES ('1-2000',$1500,'1-0000')
INSERT INTO t0 VALUES ('2-0000',$0,'')
INSERT INTO t0 VALUES ('2-1000',$0,'2-0000')
INSERT INTO t0 VALUES ('2-1100',$0,'2-1000')
INSERT INTO t0 VALUES ('2-1110',$75,'2-1100')
INSERT INTO t0 VALUES ('2-1120',-$40,'2-1100')
INSERT INTO t0 VALUES ('2-2000',$1200,'2-0000')

* get the base data into the summary cursor
SELECT * FROM t0 INTO CURSOR ts0

* declare and initialise the variables
LOCAL lnPass, lcThisTable, lcPrevTable, ;
  lcThisSum, lcPrevSum
lnPass = 0

* loop until the SQL statement doesn't return any
* rows
DO WHILE .T.

  * set up the cursor names for this pass
  * note that if you're using VFP 5 or below
  * you'll need to replace the TRAN() call with
  * ALLTRIM(STR())
  lcPrevTable = 't' + TRAN(MOD(lnPass, 2))
  lcPrevSum = 'ts' + TRAN(MOD(lnPass, 2))
  lnPass = lnPass + 1
  lcThisTable = 't' + TRAN(MOD(lnPass, 2))
  lcThisSum = 'ts' + TRAN(MOD(lnPass, 2))

  * get the parent's components from this level
  SELECT a.PAC AS AC, SUM(a.Bal) AS Bal, b.PAC AS PAC;
    FROM (lcPrevTable) a ;
    inner JOIN (lcPrevTable) b ON a.PAC == b.AC ;
    GROUP BY 1 ;
    INTO CURSOR (lcThisTable)

  * close the previous cursor - no longer required
  USE IN (lcPrevTable)

  * if there were rows returned from the summary query
  * append them to the previous summary results
  IF RECCOUNT(lcThisTable) > 0
    SELECT * ;
      FROM (lcThisTable) ;
      UNION ALL ;
      SELECT * ;
      FROM (lcPrevSum) ;
      INTO CURSOR (lcThisSum)

    * otherwise, the walk up the hierarchy's complete
  ELSE
    EXIT
  ENDIF

  * check that the number of levels walked is not
  * excessive
  IF lnPass > MAX_LEVELS
    * recursive heirarchy definition
    MESSAGEBOX(MAX_LEVELS_STRING_LOC, ;
      MB_ICONSTOP, MAX_LEVELS_CAPTION_LOC)
    CANCEL
  ENDIF
ENDDO

* clean up the tables. It's lcThisSum that needs to
* be closed because that's the empty cursor that
* triggered the exit from the loop
USE IN (lcThisSum)
USE IN (lcThisTable)

* Do the sums to get the results for each level
SELECT AC, SUM(Bal) AS Bal, PAC, ;
  COUNT(*) - 1 AS SubLevels ;
  FROM (lcPrevSum) ;
  GROUP BY 1 ;
  INTO CURSOR FinalSummary

* clean up the last of the working cursors
USE IN (lcPrevSum)

* display the results (you could get a little
* fancier here <g>)
BROW

* clean up the result set
USE IN FinalSummary
